<?php

namespace VSHM\Bus;

use VSHM\Tools;

defined('ABSPATH') || exit;

/**
 * Class Bus
 *
 * @package VSHM
 */
class Bus implements BusInterface
{

    /**
     * @var array
     */
    private $subscriptions = [];

    public const AGENT_SYSTEM = 'system';
    public const AGENT_USER   = 'user';
    public const AGENT_APP    = 'app';

    /**
     * @param string $commandClassName
     * @param string $handlerClassName
     */
    public function subscribe(string $commandClassName, string $handlerClassName): void
    {
        $this->subscriptions[ $commandClassName ] = $handlerClassName;
    }

    /**
     * @param Command     $command
     * @param string      $agent_type The type dispatcher (can be the system or user or API token or else)
     * @param string|null $agent_id   the ID of the dispatcher
     */
    public function dispatch(Command $command, string $agent_type = self::AGENT_SYSTEM, string $agent_id = NULL): void
    {
        $commandName = get_class($command);
        $handlerName = $commandName . 'Handler';
        if (!class_exists($handlerName)) {
            if (isset($this->subscriptions[ $commandName ]) && class_exists($this->subscriptions[ $commandName ])) {
                $handlerName = $this->subscriptions[ $commandName ];
            } else {
                throw new \InvalidArgumentException(sprintf('Handler class %s not found for command %s', $handlerName, $commandName));
            }
        }
        $handler = new $handlerName();
        if ($handler instanceof Handler) {
            $classNameShort = Tools::get_short_classname(get_class($command));

            do_action('vshm_bus_dispatching', $classNameShort, $command, $agent_type, $agent_id);
            do_action('vshm_dispatching_' . $classNameShort, $command, $agent_type, $agent_id);

            $handler->dispatch($command);

            do_action('vshm_dispatched_' . $classNameShort, $command, $agent_type, $agent_id);
            do_action('vshm_bus_dispatched', $classNameShort, $command, $agent_type, $agent_id);
        } else {
            throw new \InvalidArgumentException(sprintf('Class %s not an instance of Handler for command %s', $handlerName, $commandName));
        }
    }

    public function getSubscriptions(): array
    {
        return $this->subscriptions;
    }
}