<?php

namespace VSHM\Bus;

use VSHM\Modules\Notifications;
use VSHM\Providers\Customers;
use VSHM\Providers\Reservations;
use VSHM\Providers\Services;
use VSHM\Providers\ServicesData;
use VSHM\Settings\SenderEmail;
use VSHM\Settings\Service\CancellationEmailToAdmin;

defined('ABSPATH') || exit;

/**
 * SendAdminBookingCancellationEmailHandler
 *
 * @package VSHM\Bus
 */
class SendAdminBookingCancellationEmailHandler implements Handler
{
    public function dispatch(Command $command)
    {
        /** @var $command SendAdminBookingCancellationEmail */

        $reservation = Reservations::provideBy(['id' => $command->getReservationId()], TRUE);

        if (!$reservation) {
            return;
        }

        $send_to     = ServicesData::provideBy(['service_id' => $reservation['serviceId'], 'key' => CancellationEmailToAdmin::ID_SEND_TO], TRUE);
        $subject_raw = ServicesData::provideBy(['service_id' => $reservation['serviceId'], 'key' => CancellationEmailToAdmin::ID_SUBJECT], TRUE);
        $body_raw    = ServicesData::provideBy(['service_id' => $reservation['serviceId'], 'key' => CancellationEmailToAdmin::ID_BODY], TRUE);

        $customer = Customers::provideBy(['id' => $reservation['customerId']], TRUE);

        $preparedValues = Notifications::prepare_placeholders($command->getReservationId(), CancellationEmailToAdmin::ID);

        $subject = strip_tags(Notifications::find_and_replace_hooks($subject_raw, $preparedValues));
        $body    = Notifications::find_and_replace_hooks($body_raw, $preparedValues);

        $recipients = explode(',', $send_to);

        foreach ($recipients as $recipient) {
            $recipient = trim($recipient);
            if (!filter_var($recipient, FILTER_VALIDATE_EMAIL)) {
                error_log('Invalid email address: ' . $recipient);
                continue;
            }

            $from = [
                'name'    => $customer['name'],
                'address' => !(vshm()->settings()[ SenderEmail::ID ] ?? '')
                    ? get_option('admin_email')
                    : vshm()->settings()[ SenderEmail::ID ]
            ];

            $send_command = new SendEmail(
                $subject,
                $body,
                [$recipient],
                $from,
                [],
                [],
                [],
                [$customer['email']]
            );

            do_action('tbk_send_admin_booking_cancellation_email', $reservation, $send_command);

            vshm()->bus->dispatch($send_command);

        }

    }
}