<?php

namespace VSHM\Bus;

use VSHM\Modules\Notifications;
use VSHM\Providers\Customers;
use VSHM\Providers\Reservations;
use VSHM\Providers\ServiceProviders;
use VSHM\Providers\ServicesData;
use VSHM\Settings\SenderEmail;
use VSHM\Settings\Service\ConfirmationEmailToCustomer;

defined('ABSPATH') || exit;

/**
 * SendCustomerBookingConfirmationEmailHandler
 *
 * @package VSHM\Bus
 */
class SendCustomerBookingConfirmationEmailHandler implements Handler
{
    public function dispatch(Command $command)
    {
        /** @var $command SendCustomerBookingConfirmationEmail */

        $reservation = Reservations::provideBy(['id' => $command->getReservationId()], TRUE);

        if (!$reservation) {
            return;
        }

        $send_from   = ServicesData::provideBy(['service_id' => $reservation['serviceId'], 'key' => ConfirmationEmailToCustomer::ID_SEND_FROM], TRUE);
        $subject_raw = ServicesData::provideBy(['service_id' => $reservation['serviceId'], 'key' => ConfirmationEmailToCustomer::ID_SUBJECT], TRUE);
        $body_raw    = ServicesData::provideBy(['service_id' => $reservation['serviceId'], 'key' => ConfirmationEmailToCustomer::ID_BODY], TRUE);

        $subject_raw = apply_filters('tbk_send_email_pre_content', $subject_raw, ConfirmationEmailToCustomer::ID_SUBJECT, $command->getReservationId());
        $body_raw    = apply_filters('tbk_send_email_pre_content', $body_raw, ConfirmationEmailToCustomer::ID_BODY, $command->getReservationId());

        $provider = ServiceProviders::provideBy(['id' => $reservation['providerId']], TRUE);
        $customer = Customers::provideBy(['id' => $reservation['customerId']], TRUE);

        if ($send_from === ConfirmationEmailToCustomer::SEND_FROM_ADMIN) {
            $from = [
                'name'    => get_option('blogname'),
                'address' => !(vshm()->settings()[ SenderEmail::ID ] ?? '')
                    ? get_option('admin_email')
                    : vshm()->settings()[ SenderEmail::ID ]
            ];
        } else {
            $from = [
                'name'    => $provider['name'],
                'address' => $provider['email']
            ];
        }

        $preparedValues = Notifications::prepare_placeholders($command->getReservationId(), ConfirmationEmailToCustomer::ID);

        $subject = wp_strip_all_tags(Notifications::find_and_replace_hooks($subject_raw, $preparedValues));
        $body    = Notifications::find_and_replace_hooks($body_raw, $preparedValues);

        $send_command = new SendEmail(
            $subject,
            $body,
            [$customer['email']],
            $from
        );

        do_action('tbk_send_customer_booking_confirmation_email', $reservation, $send_command);

        vshm()->bus->dispatch($send_command);


    }
}