<?php

namespace VSHM\Bus;

use VSHM\Modules\Notifications;
use VSHM\Providers\Customers;
use VSHM\Providers\Files;
use VSHM\Providers\Reservations;
use VSHM\Providers\ServiceProviderCustomData;
use VSHM\Providers\ServiceProviders;
use VSHM\Providers\Services;
use VSHM\Providers\ServicesData;
use VSHM\Settings\SenderEmail;
use VSHM\Settings\Service\ConfirmationEmailToAdmin;
use VSHM\Settings\Service\Personal_ConfirmationEmailToProvider;

defined('ABSPATH') || exit;

/**
 * SendProviderBookingConfirmationEmailHandler
 *
 * @package VSHM\Bus
 */
class SendProviderBookingConfirmationEmailHandler implements Handler
{
    public function dispatch(Command $command)
    {
        /** @var $command SendProviderBookingConfirmationEmail */

        $reservation = Reservations::provideBy(['id' => $command->getReservationId()], TRUE);

        if (!$reservation) {
            return;
        }

        $send_attachments = ServiceProviderCustomData::provideBy([
            'service_id'  => $reservation['serviceId'],
            'provider_id' => $reservation['providerId'],
            'key'         => Personal_ConfirmationEmailToProvider::ID_ATTACHMENTS
        ], TRUE);
        $subject_raw      = ServiceProviderCustomData::provideBy([
            'service_id'  => $reservation['serviceId'],
            'provider_id' => $reservation['providerId'],
            'key'         => Personal_ConfirmationEmailToProvider::ID_SUBJECT
        ], TRUE);
        $body_raw         = ServiceProviderCustomData::provideBy([
            'service_id'  => $reservation['serviceId'],
            'provider_id' => $reservation['providerId'],
            'key'         => Personal_ConfirmationEmailToProvider::ID_BODY
        ], TRUE);

        $customer = Customers::provideBy(['id' => $reservation['customerId']], TRUE);
        $provider = ServiceProviders::provideBy(['id' => $reservation['providerId']], TRUE);

        $from = [
            'name'    => $customer['name'],
            'address' => !(vshm()->settings()[ SenderEmail::ID ] ?? '')
                ? get_option('admin_email')
                : vshm()->settings()[ SenderEmail::ID ]
        ];

        $attachments = [];

        if ($send_attachments) {
            $files = Files::provideByReservationId($command->getReservationId());
            foreach ($files as $hash => $file) {
                $attachments[] = $file['file'];
            }
        }

        $preparedValues = Notifications::prepare_placeholders($command->getReservationId(), Personal_ConfirmationEmailToProvider::ID);

        $subject = wp_strip_all_tags(Notifications::find_and_replace_hooks($subject_raw, $preparedValues));
        $body    = Notifications::find_and_replace_hooks($body_raw, $preparedValues);

        $send_command = new SendEmail(
            $subject,
            $body,
            [$provider['email']],
            $from,
            $attachments,
            [],
            [],
            [$customer['email']]
        );

        do_action('tbk_send_provider_booking_confirmation_email', $reservation, $send_command);

        vshm()->bus->dispatch($send_command);
    }
}