<?php

namespace VSHM\Modules\Gcal3Way;

use VSHM\Bus\Hook;
use VSHM\Modules\Gcal2Ways;
use VSHM\Providers\Reservations;
use VSHM\Providers\ReservationsData;
use VSHM\Providers\ServiceProviders;
use VSHM\Settings\Reservation\AvailabilityId;
use VSHM\Settings\Reservation\GoogleCalendarEventId;

defined('ABSPATH') || exit;

class AddCustomerToEvent extends Hook
{

    public static function hook(): void
    {
        add_action('vshm_dispatched_ChangeReservationCustomer', [self::class, 'callback']);
    }

    public static function callback($command): void
    {
        /** @var $command  \VSHM\Bus\ChangeReservationCustomer */
        $reservation = Reservations::provideBy(['id' => $command->getId()], TRUE);

        if (!$reservation) {
            return;
        }

        $originCalendar = ReservationsData::provideBy(['reservation_id' => $command->getId(), 'key' => AvailabilityId::ID], TRUE);
        $gcalEventId    = ReservationsData::provideBy(['reservation_id' => $command->getId(), 'key' => GoogleCalendarEventId::ID], TRUE);

        if (!$gcalEventId || !$originCalendar) {
            return;
        }

        $provider = ServiceProviders::provideBy(['id' => $reservation['providerId']], TRUE);

        $destination = Gcal2Ways::get_destination_from_availability_id($originCalendar, $provider);

        $otherReservations = Gcal2Ways::_are_there_other_reservations($reservation['id']);
        if (!$otherReservations) {
            $otherReservations = [$command->getId()];
        } else {
            $otherReservations[] = $command->getId();
        }
        OperateEvent::addReservation([
            'reservations' => $otherReservations,
            'provider_id'  => $reservation['providerId'],
            'service_id'   => $reservation['serviceId'],
            'customer_id'  => $command->getCustomerId(),
            'eventId'      => $gcalEventId,
            'destination'  => $destination
        ]);
    }
}