<?php

namespace VSHM\Modules\Gcal3Way;

use VSHM\Bus\ChangeReservationStatus;
use VSHM\Bus\Command;
use VSHM\Bus\CreateReservation;
use VSHM\Bus\Hook;
use VSHM\Bus\UpdateOrCreateReservationProperty;
use VSHM\Modules\Gcal2Ways;
use VSHM\Providers\Reservations;
use VSHM\Providers\ReservationsData;
use VSHM\Providers\ServiceProviderCustomData;
use VSHM\Providers\ServiceProviders;
use VSHM\Providers\Services;
use VSHM\Settings\AllowCart;
use VSHM\Settings\Provider\GoogleApiToken;
use VSHM\Settings\Reservation\AvailabilityId;
use VSHM\Settings\Reservation\GoogleCalendarEventId;
use VSHM\Settings\Reservation\GoogleCalendarId;
use VSHM\Settings\Service\Personal_GcalCreateEvent;

defined('ABSPATH') || exit;

class CreateEvent extends Hook
{
    public static function hook(): void
    {
        add_action('vshm_dispatched_CreateReservation', [self::class, 'callbackExtended'], 10, 3);
        add_action('vshm_dispatching_ChangeReservationStatus', [self::class, 'callbackExtended'], 10, 3);
    }

    public static function callbackExtended(Command $command, $agent_type, $agent_user): void
    {
        // TODO: @NEXT, more reliable system to discriminate automated workflows
        if ($command instanceof ChangeReservationStatus && $agent_type !== \VSHM\Bus\Bus::AGENT_SYSTEM) {
            return;
        }
        $reservation_id = NULL;
        $providerId     = NULL;
        $serviceId      = NULL;
        $start          = NULL;
        $end            = NULL;
        $customerId     = NULL;
        if ($command instanceof CreateReservation) {
            $reservation_id = $command->getId();
            $providerId     = $command->getProviderId();
            $serviceId      = $command->getServiceId();
            $start          = $command->getStart();
            $end            = $command->getEnd();
            $customerId     = $command->getUserId();
        } elseif ($command instanceof ChangeReservationStatus) {
            $reservation_id = $command->getId();
            $reservation    = Reservations::provideBy(['id' => $command->getId()], TRUE);
            $providerId     = $reservation['providerId'];
            $serviceId      = $reservation['serviceId'];
            $start          = $reservation['start'];
            $end            = $reservation['end'];
            $customerId     = $reservation['customerId'];
        }
        $originCalendar = ReservationsData::provideBy(['reservation_id' => $reservation_id, 'key' => AvailabilityId::ID], TRUE);
        $gcalEventId    = ReservationsData::provideBy(['reservation_id' => $reservation_id, 'key' => GoogleCalendarEventId::ID], TRUE);
        $provider       = ServiceProviders::provideBy(['id' => $providerId], TRUE);
        $service        = Services::provideBy(['id' => $serviceId], TRUE);

        if (!filter_var(ServiceProviderCustomData::provideBy(['provider_id' => $providerId, 'service_id' => $serviceId, 'key' => Personal_GcalCreateEvent::ID], TRUE), FILTER_VALIDATE_BOOLEAN)) {
            return;
        }

        $destination = Gcal2Ways::get_destination_from_availability_id($originCalendar, $provider);

        if ($originCalendar
            && $provider
            && $destination
            && $provider[ GoogleApiToken::ID ]
            && $service
            && $service['class'] !== 'unscheduled'
            && !$gcalEventId
            && $command->getStatus() === 'confirmed'
        ) {
            $to_be_created = [];
            $to_be_updated = [];
            $create        = TRUE;

            $otherReservations = Gcal2Ways::_are_there_other_reservations($reservation_id);

            /*
             * @NEXT: at the moment, if multiple slot selection is active, one event per reservation is created to keep things simple.
             */
            if (!empty($otherReservations) && !vshm()->settings()[ AllowCart::ID ]) {
                $existingEventId = ReservationsData::provideBy(['reservation_id' => $otherReservations[0], 'key' => GoogleCalendarEventId::ID], TRUE);
                if ($existingEventId) {
                    $create = FALSE;
                }
            }

            if ($create) {
                OperateEvent::create([
                    'reservation_id' => $reservation_id,
                    'provider_id'    => $providerId,
                    'service_id'     => $serviceId,
                    'customer_id'    => $customerId,
                    'start'          => $start,
                    'end'            => $end,
                    'destination'    => $destination,
                    'reservations'   => [$reservation_id]
                ]);
            } else {
                $otherReservations[] = $reservation_id;
                OperateEvent::addReservation([
                    'reservations' => $otherReservations,
                    'provider_id'  => $providerId,
                    'service_id'   => $serviceId,
                    'customer_id'  => $customerId,
                    'destination'  => $destination,
                    'eventId'      => $existingEventId
                ]);
                vshm()->bus->dispatch(new UpdateOrCreateReservationProperty($reservation_id, GoogleCalendarEventId::ID, $existingEventId));
                vshm()->bus->dispatch(new UpdateOrCreateReservationProperty($reservation_id, GoogleCalendarId::ID, $destination));
            }

        }
    }
}