<?php

namespace VSHM\Modules\Gcal3Way;

use VSHM\Bus\CancelReservation;
use VSHM\Bus\Command;
use VSHM\Bus\DeleteReservation;
use VSHM\Bus\DeleteReservationProperty;
use VSHM\Bus\Hook;
use VSHM\Modules\Gcal2Ways;
use VSHM\Providers\Reservations;
use VSHM\Providers\ReservationsData;
use VSHM\Providers\ServiceProviders;
use VSHM\Providers\Services;
use VSHM\Settings\Provider\GoogleApiToken;
use VSHM\Settings\Reservation\AvailabilityId;
use VSHM\Settings\Reservation\GoogleCalendarEventId;
use VSHM\Settings\Reservation\GoogleCalendarId;

defined('ABSPATH') || exit;

class DeleteEvent extends Hook
{
    public static function hook(): void
    {
        add_action('vshm_dispatching_DeleteReservation', [self::class, 'callback']);
        add_action('vshm_dispatched_CancelReservation', [self::class, 'callback']);
    }

    public static function callback(Command $command): void
    {
        /** @var DeleteReservation|CancelReservation $command */
        $reservation    = Reservations::provideBy(['id' => $command->getId()], TRUE);
        $originCalendar = ReservationsData::provideBy(['reservation_id' => $command->getId(), 'key' => AvailabilityId::ID], TRUE);
        $gcalEventId    = ReservationsData::provideBy(['reservation_id' => $command->getId(), 'key' => GoogleCalendarEventId::ID], TRUE);
        $provider       = ServiceProviders::provideBy(['id' => $reservation['providerId']], TRUE);
        $service        = Services::provideBy(['id' => $reservation['serviceId']], TRUE);

        $destination = Gcal2Ways::get_destination_from_availability_id($originCalendar, $provider);

        if ($originCalendar
            && $provider
            && $destination
            && $provider[ GoogleApiToken::ID ]
            && $service
            && $gcalEventId
        ) {

            if ($otherReservations = Gcal2Ways::_are_there_other_reservations_by_event_id($command->getId())) {
                OperateEvent::removeReservation([
                    'provider_id'  => $reservation['providerId'],
                    'customer_id'  => $reservation['customerId'],
                    'service_id'   => $reservation['serviceId'],
                    'destination'  => $destination,
                    'eventId'      => $gcalEventId,
                    'reservations' => $otherReservations
                ]);
                vshm()->bus->dispatch(new DeleteReservationProperty($command->getId(), GoogleCalendarEventId::ID));
                vshm()->bus->dispatch(new DeleteReservationProperty($command->getId(), GoogleCalendarId::ID));
            } else {
                OperateEvent::remove([
                    'provider_id' => $reservation['providerId'],
                    'destination' => $destination,
                    'eventId'     => $gcalEventId
                ]);
            }

        }
    }
}