<?php

namespace VSHM\Modules\Gcal3Way;

use VSHM\Bus\ChangeReservationDate;
use VSHM\Bus\ChangeReservationProvider;
use VSHM\Bus\ChangeReservationService;
use VSHM\Bus\Command;
use VSHM\Bus\DeleteReservationProperty;
use VSHM\Bus\Hook;
use VSHM\Bus\UpdateOrCreateReservationProperty;
use VSHM\Modules\Gcal2Ways;
use VSHM\Plugin\DateTimeTbk;
use VSHM\Providers\Reservations;
use VSHM\Providers\ReservationsData;
use VSHM\Providers\ServiceProviderCustomData;
use VSHM\Providers\ServiceProviders;
use VSHM\Providers\Services;
use VSHM\Settings\Provider\GoogleApiToken;
use VSHM\Settings\Provider\GoogleCalendars;
use VSHM\Settings\Reservation\AvailabilityId;
use VSHM\Settings\Reservation\GoogleCalendarEventId;
use VSHM\Settings\Reservation\GoogleCalendarId;
use VSHM\Settings\Service\Personal_EventTitleBooked;
use VSHM\Tools;

defined('ABSPATH') || exit;

class EditEvent extends Hook
{
    public static function hook(): void
    {
        add_action('vshm_dispatching_ChangeReservationService', [self::class, 'callback']);
        add_action('vshm_dispatching_ChangeReservationProvider', [self::class, 'callback']);
        add_action('vshm_dispatching_ChangeReservationDate', [self::class, 'callback']);
    }

    public static function callback(Command $command): void
    {
        switch (TRUE) {
            case $command instanceof ChangeReservationService:
                $reservation    = Reservations::provideBy(['id' => $command->getId()], TRUE);
                $originCalendar = ReservationsData::provideBy(['reservation_id' => $command->getId(), 'key' => AvailabilityId::ID], TRUE);
                $gcalEventId    = ReservationsData::provideBy(['reservation_id' => $command->getId(), 'key' => GoogleCalendarEventId::ID], TRUE);

                if (!$gcalEventId || !$originCalendar) {
                    return;
                }

                $provider = ServiceProviders::provideBy(['id' => $reservation['providerId']], TRUE);
                $service  = Services::provideBy(['id' => $command->getServiceId()], TRUE);

                $destination = Gcal2Ways::get_destination_from_availability_id($originCalendar, $provider);

                if (!$provider || !$service || !$destination || !$provider[ GoogleApiToken::ID ]) {
                    return;
                }

                if ($service['class'] === 'unscheduled') {
                    if ($otherReservations = Gcal2Ways::_are_there_other_reservations($command->getId())) {
                        OperateEvent::removeReservation([
                            'provider_id'  => $reservation['providerId'],
                            'customer_id'  => $reservation['customerId'],
                            'service_id'   => $reservation['serviceId'], //old service!
                            'destination'  => $destination,
                            'eventId'      => $gcalEventId,
                            'reservations' => $otherReservations
                        ]);
                    } else {
                        OperateEvent::remove([
                            'provider_id' => $reservation['providerId'],
                            'destination' => $destination,
                            'eventId'     => $gcalEventId
                        ]);
                    }
                    vshm()->bus->dispatch(new DeleteReservationProperty($command->getId(), GoogleCalendarEventId::ID));
                    vshm()->bus->dispatch(new DeleteReservationProperty($command->getId(), GoogleCalendarId::ID));

                } else {
                    if ($otherReservations = Gcal2Ways::_are_there_other_reservations($command->getId())) {
                        OperateEvent::removeReservation([
                            'provider_id'  => $reservation['providerId'],
                            'customer_id'  => $reservation['customerId'],
                            'service_id'   => $reservation['serviceId'], //old service!
                            'destination'  => $destination,
                            'eventId'      => $gcalEventId,
                            'reservations' => $otherReservations
                        ]);

                        $newOriginCalendar = NULL;
                        $newDestination    = NULL;
                        foreach ($provider[ GoogleCalendars::ID ] as $calendarId => $calendar) {
                            if (in_array($command->getServiceId(), $calendar['services'], TRUE)) {
                                $newOriginCalendar = $calendarId;
                                $newDestination    = $calendar[ Gcal2Ways::CALENDAR_DESTINATION ] ?? NULL;
                                break;
                            }
                        }

                        if ($newOriginCalendar) {
                            vshm()->bus->dispatch(new UpdateOrCreateReservationProperty($command->getId(), AvailabilityId::ID, $newOriginCalendar));

                            if ($newDestination) {
                                if ($otherReservations = Gcal2Ways::_are_there_other_reservations($command->getId())) {
                                    $newGcalEventId      = ReservationsData::provideBy(['reservation_id' => $otherReservations[0]['id'], 'key' => GoogleCalendarEventId::ID], TRUE);
                                    $otherReservations[] = $command->getId();
                                    OperateEvent::addReservation([
                                        'provider_id'  => $reservation['providerId'],
                                        'customer_id'  => $reservation['customerId'],
                                        'service_id'   => $command->getServiceId(),
                                        'destination'  => $newDestination,
                                        'eventId'      => $newGcalEventId,
                                        'reservations' => $otherReservations
                                    ]);
                                    vshm()->bus->dispatch(new UpdateOrCreateReservationProperty($command->getId(), GoogleCalendarId::ID, $newDestination));
                                    vshm()->bus->dispatch(new UpdateOrCreateReservationProperty($command->getId(), GoogleCalendarEventId::ID, $newGcalEventId));
                                } else {
                                    OperateEvent::create([
                                        'provider_id'    => $reservation['providerId'],
                                        'reservation_id' => $command->getId(),
                                        'service_id'     => $command->getServiceId(),
                                        'customer_id'    => $reservation['customerId'],
                                        'destination'    => $newDestination,
                                        'start'          => $reservation['start'],
                                        'end'            => $reservation['end'],
                                        'reservations'   => [$reservation['id']]
                                    ]);
                                }
                            }

                        }


                    } else {
                        try {
                            $client = Gcal2Ways::_client();
                            $client->setAccessToken($provider[ GoogleApiToken::ID ]);
                            $g_service = new \Google\Service\Calendar($client);
                            $event     = $g_service->events->get($destination, $gcalEventId);
                            if ($event instanceof \Google\Service\Calendar\Event) {
                                $eventSummary = ServiceProviderCustomData::provideBy([
                                    'provider_id' => $reservation['providerId'],
                                    'service_id'  => $command->getServiceId(),
                                    'key'         => Personal_EventTitleBooked::ID
                                ], TRUE) ?? '';
                                $event->setSummary(Gcal2Ways::parseDynamicHooks($eventSummary, $command->getId()));
                                $event->setTransparency('opaque');
                                $g_service->events->update($destination, $gcalEventId, $event);
                            }
                        } catch (\Exception $e) {
                            // Skipping
                            Tools::log_dump($e->getMessage());
                        }
                    }
                }

                break;
            case $command instanceof ChangeReservationDate:
                $reservation    = Reservations::provideBy(['id' => $command->getId()], TRUE);
                $originCalendar = ReservationsData::provideBy(['reservation_id' => $command->getId(), 'key' => AvailabilityId::ID], TRUE);
                $gcalEventId    = ReservationsData::provideBy(['reservation_id' => $command->getId(), 'key' => GoogleCalendarEventId::ID], TRUE);

                if (!$gcalEventId || !$originCalendar) {
                    return;
                }

                $provider = ServiceProviders::provideBy(['id' => $reservation['providerId']], TRUE);
                $service  = Services::provideBy(['id' => $reservation['serviceId']], TRUE);

                $destination = Gcal2Ways::get_destination_from_availability_id($originCalendar, $provider);

                if (!$provider || !$service || !$destination || !$provider[ GoogleApiToken::ID ]) {
                    return;
                }

                if ($otherReservations = Gcal2Ways::_are_there_other_reservations($command->getId())) {

                    OperateEvent::removeReservation([
                        'provider_id'  => $reservation['providerId'],
                        'customer_id'  => $reservation['customerId'],
                        'service_id'   => $reservation['serviceId'],
                        'destination'  => $destination,
                        'eventId'      => $gcalEventId,
                        'reservations' => $otherReservations
                    ]);

                    if ($otherReservations = Gcal2Ways::_are_there_other_reservations($command->getId())) {
                        $newGcalEventId      = ReservationsData::provideBy(['reservation_id' => $otherReservations[0]['id'], 'key' => GoogleCalendarEventId::ID], TRUE);
                        $otherReservations[] = $command->getId();
                        OperateEvent::addReservation([
                            'provider_id'  => $reservation['providerId'],
                            'customer_id'  => $reservation['customerId'],
                            'service_id'   => $reservation['serviceId'],
                            'destination'  => $destination,
                            'eventId'      => $newGcalEventId,
                            'reservations' => $otherReservations
                        ]);
                        vshm()->bus->dispatch(new UpdateOrCreateReservationProperty($command->getId(), GoogleCalendarEventId::ID, $newGcalEventId));
                    } else {
                        OperateEvent::create([
                            'provider_id'    => $reservation['providerId'],
                            'reservation_id' => $command->getId(),
                            'service_id'     => $reservation['serviceId'],
                            'customer_id'    => $reservation['customerId'],
                            'destination'    => $destination,
                            'start'          => $command->getReference() === 'start' ? $command->getUnixTime() : $reservation['start'],
                            'end'            => $command->getReference() === 'end' ? $command->getUnixTime() : $reservation['end'],
                            'reservations'   => [$reservation['id']]
                        ]);
                    }


                } else {
                    $client = Gcal2Ways::_client();
                    $client->setAccessToken($provider[ GoogleApiToken::ID ]);
                    $g_service = new \Google\Service\Calendar($client);

                    try {
                        $event = $g_service->events->get($destination, $gcalEventId);
                        if ($event instanceof \Google\Service\Calendar\Event) {
                            $date = new \Google\Service\Calendar\EventDateTime();
                            $date->setDateTime(DateTimeTbk::createFromFormatSilently('U', $command->getUnixTime())->format(\DATE_RFC3339));
                            if ($command->getReference() === 'start') {
                                $event->setStart($date);
                            } else {
                                $event->setEnd($date);
                            }
                            $event->setTransparency('opaque');
                            $g_service->events->update($destination, $gcalEventId, $event);
                        }
                    } catch (\Exception $e) {
                        // Skipping
                        Tools::log_dump($e->getMessage());
                    }
                }

                break;

            case $command instanceof ChangeReservationProvider:
                $reservation    = Reservations::provideBy(['id' => $command->getId()], TRUE);
                $originCalendar = ReservationsData::provideBy(['reservation_id' => $command->getId(), 'key' => AvailabilityId::ID], TRUE);
                $gcalEventId    = ReservationsData::provideBy(['reservation_id' => $command->getId(), 'key' => GoogleCalendarEventId::ID], TRUE);

                if (!$gcalEventId || !$originCalendar) {
                    return;
                }

                $provider     = ServiceProviders::provideBy(['id' => $command->getProviderId()], TRUE);
                $old_provider = ServiceProviders::provideBy(['id' => $reservation['providerId']], TRUE);
                $service      = Services::provideBy(['id' => $reservation['serviceId']], TRUE);

                $destination = Gcal2Ways::get_destination_from_availability_id($originCalendar, $old_provider);

                if (!$old_provider || !$service || !$destination || !$old_provider[ GoogleApiToken::ID ]) {
                    return;
                }

                /**
                 * Removing old provider Gcal event
                 */
                if ($otherReservations = Gcal2Ways::_are_there_other_reservations($command->getId())) {
                    OperateEvent::removeReservation([
                        'provider_id'  => $old_provider['id'],
                        'destination'  => $destination,
                        'eventId'      => $gcalEventId,
                        'service_id'   => $service['id'],
                        'customer_id'  => $reservation['customerId'],
                        'reservations' => $otherReservations
                    ]);

                } else {
                    try {
                        OperateEvent::remove([
                            'provider_id' => $old_provider['id'],
                            'eventId'     => $gcalEventId,
                            'destination' => $destination,
                        ]);
                    } catch (\Exception $e) {
                        // Skipping
                        Tools::log_dump($e->getMessage());
                    }
                }
                vshm()->bus->dispatch(new DeleteReservationProperty($command->getId(), GoogleCalendarEventId::ID));
                vshm()->bus->dispatch(new DeleteReservationProperty($command->getId(), GoogleCalendarId::ID));
                vshm()->bus->dispatch(new DeleteReservationProperty($command->getId(), AvailabilityId::ID));

                /**
                 * Adding new provider Gcal event
                 */
                if (!$provider || !$provider[ GoogleApiToken::ID ] || !isset($provider[ GoogleCalendars::ID ])) {
                    return;
                }
                foreach ($provider[ GoogleCalendars::ID ] as $calendarId => $calendar) {
                    if (in_array($reservation['serviceId'], $calendar['services'], TRUE)) {
                        vshm()->bus->dispatch(new UpdateOrCreateReservationProperty($command->getId(), AvailabilityId::ID, $calendarId));

                        if ($calendar[ Gcal2Ways::CALENDAR_DESTINATION ]) {
                            if ($otherReservations = Gcal2Ways::_are_there_other_reservations($command->getId())) {
                                $newGcalEventId      = ReservationsData::provideBy(['reservation_id' => $otherReservations[0]['id'], 'key' => GoogleCalendarEventId::ID], TRUE);
                                $otherReservations[] = $command->getId();
                                OperateEvent::addReservation([
                                    'provider_id'  => $command->getProviderId(),
                                    'customer_id'  => $reservation['customerId'],
                                    'service_id'   => $reservation['serviceId'],
                                    'destination'  => $calendar[ Gcal2Ways::CALENDAR_DESTINATION ],
                                    'eventId'      => $newGcalEventId,
                                    'reservations' => $otherReservations
                                ]);
                                vshm()->bus->dispatch(new UpdateOrCreateReservationProperty($command->getId(), GoogleCalendarEventId::ID, $newGcalEventId));
                            } else {
                                OperateEvent::create([
                                    'reservation_id' => $command->getId(),
                                    'provider_id'    => $command->getProviderId(),
                                    'service_id'     => $reservation['serviceId'],
                                    'customer_id'    => $reservation['customerId'],
                                    'start'          => $reservation['start'],
                                    'end'            => $reservation['end'],
                                    'destination'    => $calendar[ Gcal2Ways::CALENDAR_DESTINATION ],
                                    'reservations'   => [$reservation['id']]
                                ]);
                            }
                        }

                        break;
                    }
                }

                break;
        }
    }
}