<?php

namespace VSHM\Modules\Gcal3Way;

use VSHM\Bus\DeleteService;
use VSHM\Bus\Hook;
use VSHM\Bus\UpdateOrCreateServiceProperty;
use VSHM\Bus\UpdateProviderProperty;
use VSHM\Providers\ServiceProviders;
use VSHM\Settings\Provider\AllowedServices;
use VSHM\Settings\Provider\GoogleCalendars;

defined('ABSPATH') || exit;

class ServicesCleanup extends Hook
{

    public static function hook(): void
    {
        add_action('vshm_dispatched_DeleteService', [self::class, 'callback']);
        add_action('vshm_dispatched_UpdateOrCreateServiceProperty', [self::class, 'callback']);
        add_action('vshm_dispatched_UpdateProviderProperty', [self::class, 'callback']);
    }

    public static function callback($command): void
    {
        $providers = ServiceProviders::provide();

        if ($command instanceof DeleteService) {
            foreach ($providers as $provider) {
                $calendars = $provider[ GoogleCalendars::ID ];
                foreach ($calendars as $key => $calendar) {
                    if (!isset($calendar['services'])) {
                        continue;
                    }
                    $service_key = array_search($command->getId(), $calendar['services'], TRUE);
                    if ($service_key !== FALSE) {
                        unset($calendars[ $key ]['services'][ $service_key ]);
                        vshm()->bus->dispatch(new UpdateProviderProperty($provider['id'], GoogleCalendars::ID, $calendars));
                    }
                }
            }
        } elseif ($command instanceof UpdateOrCreateServiceProperty) {
            if ($command->getKey() === 'class' && $command->getValue() === 'unscheduled') {
                foreach ($providers as $provider) {
                    $calendars = $provider[ GoogleCalendars::ID ];
                    foreach ($calendars as $key => $calendar) {
                        if (!isset($calendar['services'])) {
                            continue;
                        }
                        $service_key = array_search($command->getServiceId(), $calendar['services'], TRUE);
                        if ($service_key !== FALSE) {
                            unset($calendars[ $key ]['services'][ $service_key ]);
                            vshm()->bus->dispatch(new UpdateProviderProperty($provider['id'], GoogleCalendars::ID, $calendars));
                        }
                    }
                }
            }
        } elseif ($command instanceof UpdateProviderProperty) {
            if ($command->getKey() === AllowedServices::ID) {
                foreach ($providers as $provider) {
                    if ($provider['id'] === $command->getProviderId()) {
                        $calendars = $provider[ GoogleCalendars::ID ];
                        if (is_array($calendars)) {
                            foreach ($calendars as $key => $calendar) {
                                if (!isset($calendar['services'])) {
                                    continue;
                                }
                                $calendars[ $key ]['services'] = array_values(array_intersect($command->getValue(), $calendar['services']));
                            }
                            vshm()->bus->dispatch(new UpdateProviderProperty($provider['id'], GoogleCalendars::ID, $calendars));
                        }
                    }
                }
            }
        }
    }
}