<?php

namespace VSHM\Routes;

use VSHM\Functions;
use VSHM\REST_Controller;

defined('ABSPATH') || exit;

/**
 * Class FunctionsRoute
 *
 * @package VSHM\Routes
 */
final class FunctionsRoute implements Route
{
    /**
     * @var string
     */
    private static $path = '/backend/functions/';

    public static function register(): void
    {
        REST_Controller::register_routes([
            self::$path . 'get/token/'   => [
                'methods'             => \WP_REST_Server::READABLE,
                'callback'            => function (\WP_REST_Request $request) {
                    return REST_Controller::get_ok_response(self::$path . 'get/token/', ['data' => \VSHM\Tools::generate_token()]);
                },
                'permission_callback' => static function () {
                    return Functions::current_user_can_admin();
                }
            ],
            self::$path . 'add/to/cart/' => [
                'methods'  => \WP_REST_Server::CREATABLE,
                'callback' => function (\WP_REST_Request $request) {

                    $slotId = $request->get_param('slotId');

                    if (!$slotId) {
                        return REST_Controller::get_error_response(self::$path . 'add/to/cart/', ['message' => 'Slot ID not found.'], 404);
                    }

                    $sessionSlots = vshm()->sessions->get('slots', []);

                    if (!is_array($sessionSlots)) {
                        return REST_Controller::get_error_response(self::$path . 'add/to/cart/', ['message' => 'Invalid slots session data.']);
                    }

                    $sessionSlots[] = $slotId;
                    vshm()->sessions->set('slots', $sessionSlots);

                    do_action('tbk_set_cart_cookies', TRUE);

                    return REST_Controller::get_ok_response(self::$path . 'add/to/cart/', ['slots' => $sessionSlots]);
                }
            ]
        ]);
    }

    public static function getPath(): string
    {
        return self::$path;
    }
}