<?php

namespace VSHM\Routes\Reservations;

use VSHM\Bus\DeleteReservation;
use VSHM\Functions;
use VSHM\REST_Controller;
use VSHM\Routes\ReservationsRoute;
use VSHM\Routes\SingleRoute;

defined('ABSPATH') || exit;

/**
 * Class RemoveMulti
 *
 * @package VSHM\Routes\Reservations
 */
class RemoveMulti implements SingleRoute
{
    public static function getPath(): string
    {
        return ReservationsRoute::getPath() . 'remove/multi/';
    }

    public static function get(): array
    {
        return [
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => function (\WP_REST_Request $request) {

                $item = $request->get_param('data');

                if (isset($item['ids']) && is_array($item['ids'])) {
                    foreach ($item['ids'] as $id) {
                        vshm()->bus->dispatch(new DeleteReservation($id), \VSHM\Bus\Bus::AGENT_USER, get_current_user_id());
                    }
                }

                return REST_Controller::get_ok_response(self::getPath(), [
                    'reservations' => ReservationsRoute::prepare_for_frontend()
                ]);
            },
            'args'                => [
                'data' => [
                    'required' => TRUE
                ]
            ],
            'permission_callback' => static function () {
                return Functions::current_user_can_admin(TRUE);
            }
        ];
    }
}