<?php

namespace VSHM\Settings;

use VSHM\UI\Admin\Alert;
use VSHM\UI\Admin\Element_Setting;
use VSHM\UI\Admin\Settings_Option;

defined('ABSPATH') || exit;

/**
 * Class FrontendBookingManagerPage
 *
 * @package VSHM
 */
class FrontendBookingManagerPage extends SettingBase
{

    public const ID = 'bookingManagerPage';

    public static function getBackendElement(): Element_Setting
    {
        $args  = [
            'sort_order'   => 'asc',
            'sort_column'  => 'post_title',
            'hierarchical' => 1,
            'exclude'      => '',
            'include'      => '',
            'meta_key'     => '',
            'meta_value'   => '',
            'authors'      => '',
            'exclude_tree' => '',
            'number'       => '',
            'offset'       => 0,
            'post_type'    => 'page',
            'post_status'  => 'publish'
        ];
        $pages = get_pages($args);

        $element = \VSHM\UI\Admin\Settings_Select::get(__('Reservation status page', 'team-booking'), self::ID);
        $element->setDescription(__('Select the frontend page where customers are supposed to see their reservations and to manage them.', 'team-booking'));
        $element->setAlert(Alert::warning(__('In order to work, the selected page must contain the plugin widget somewhere', 'team-booking')));

        $element->addOption(Settings_Option::get(__('No page selected', 'team-booking'), 0));
        foreach ($pages as $page) {
            $element->addOption(Settings_Option::get($page->post_title, $page->ID));
        }

        return $element;
    }

    public static function getDefault(): int
    {
        return 0;
    }

    public static function whitelist($value, string $version): int
    {
        return (int)$value;
    }
}