<?php

namespace VSHM\Settings\Service;

use VSHM\UI\Admin\Alert;
use VSHM\UI\Admin\Element_Setting;
use VSHM\UI\Admin\Settings_Dependency;

defined('ABSPATH') || exit;

/**
 * Class AssignmentRule
 *
 * @package VSHM
 */
class AssignmentRule extends ServiceSettingBase
{
    public const ID = 'assignmentRule';

    public const EQUAL  = 'equal';
    public const DIRECT = 'direct';
    public const RANDOM = 'random';

    public static function getBackendElement(): Element_Setting
    {
        $element = \VSHM\UI\Admin\Settings_Radios::get(__('Assignment rule', 'team-booking'), self::ID);
        $element->setDescription(__('In case of multiple service providers, specify which assignment rule the plugin should follow after each reservation.', 'team-booking'));
        $option = \VSHM\UI\Admin\Settings_Option::get(__('Equal', 'team-booking'), self::EQUAL);
        $option->setDescription(__('A new reservation is assigned to the provider with less of them assigned (in the service context). Based on reservations that are present in the database.', 'team-booking'));
        $element->addOption($option);
        $option = \VSHM\UI\Admin\Settings_Option::get(__('Direct', 'team-booking'), self::DIRECT);
        $option->setDescription(__('Always assign to a specific provider.', 'team-booking'));
        $element->addOption($option);
        $option = \VSHM\UI\Admin\Settings_Option::get(__('Random', 'team-booking'), self::RANDOM);
        $option->setDescription(__('Picks a random provider.', 'team-booking'));
        $element->addOption($option);

        $element->addDependency(Settings_Dependency::EQUAL('class', 'unscheduled'));

        return $element;
    }

    public static function getDefault(): string
    {
        return self::EQUAL;
    }
}