<?php

namespace VSHM\Settings\Service;

use VSHM\UI\Admin\Alert;
use VSHM\UI\Admin\Element_Setting;
use VSHM\UI\Admin\Settings_Dependency;

defined('ABSPATH') || exit;

/**
 * Class Personal_GcalAddGuests
 *
 * @package VSHM
 */
class Personal_GcalAddGuests extends ServicePersonalSettingBase
{
    public const ID = 'gcalAddGuests';

    public static function getBackendElement(): Element_Setting
    {
        $element = \VSHM\UI\Admin\Settings_Toggle::get(__('Add customers as guests of the Google Calendar event', 'team-booking'), self::ID);
        $element->setDescription(__('If active, customers receive Google notifications according to your Google Calendar settings, a copy of the event will be created in their Google Calendars (if they have it) and they will be able to read the event description. They will not be able to see the guests list.', 'team-booking'));
        $element->setAlert(Alert::warning(__('According to personal data regulations you may be required to provide a full disclosure to the customer about where their data will be stored. You should be also able to erase such data at any time if requested. Please consult the site administrator before set this option in a way that customer personal data could be stored outside this site.', 'team-booking')));
        $element->addDependency(Settings_Dependency::NOT_EQUAL('class', 'unscheduled'));
        $element->addDependency(Settings_Dependency::TRUTHY(Personal_GcalCreateEvent::ID));

        return $element;
    }

    public static function getDefault(): bool
    {
        return FALSE;
    }

    public static function whitelist($value, string $version)
    {
        return filter_var($value, FILTER_VALIDATE_BOOLEAN);
    }
}