<?php

namespace VSHM\Settings;

use VSHM\UI\Admin\Element_Setting;

defined('ABSPATH') || exit;

/**
 * Interface Setting
 *
 * Defines an interface for a generic backend setting.
 *
 * @package VSHM
 */
interface Setting
{

    /**
     * Returns the backend UI element for this setting.
     *
     * @return Element_Setting
     */
    public static function getBackendElement(): ?Element_Setting;

    /**
     * Returns the default value of the setting.
     *
     * @return mixed
     */
    public static function getDefault();

    /**
     * Subscribes the setting (actions & filters).
     *
     * @return void
     */
    public static function subscribe(): void;

    /**
     * If not public, it won't be included in the frontend settings object.
     *
     * @return boolean
     */
    public static function isPublic(): bool;

}
