<?php

namespace VSHM\Settings;

use VSHM\Bus\SaveSettings;

defined('ABSPATH') || exit;

/**
 * Implements some common setting methods.
 *
 * Defines an interface for a generic backend setting.
 *
 * @package VSHM
 */
abstract class SettingBase implements Setting
{
    public static function subscribe(): void
    {
        add_filter('vshm_default_plugin_settings', static function ($defaults) {
            $defaults[ static::ID ] = static::getDefault();

            return $defaults;
        });
        add_filter('vshm_export_settings', static function ($settings) {
            $settings[ static::ID ] = vshm()->settings()[ static::ID ];

            return $settings;
        });
        add_filter('vshm_import_settings', static function ($toSave, $settings, $version) {
            if (isset($settings[ static::ID ]) || array_key_exists(static::ID, $settings)) {
                $toSave[ static::ID ] = static::whitelist($settings[ static::ID ], $version);
            }

            return $toSave;
        }, 10, 3);
        add_action('vshm_dispatching_SaveSettings', static function (SaveSettings $command) {
            $settings = $command->getSettings();

            if (isset($settings[ static::ID ]) || array_key_exists(static::ID, $settings)) {
                $settings[ static::ID ] = static::whitelist($settings[ static::ID ], vshm()->plugin['VERSION']);
                $command->setSettings($settings);
            }
        });
    }

    public static function whitelist($value, string $version)
    {
        return $value;
    }

    public static function isPublic(): bool
    {
        return FALSE;
    }
}
