<?php

namespace VSHM\Settings;

use VSHM\UI\Admin\Element_Setting;

defined('ABSPATH') || exit;

/**
 * Implements some common setting methods.
 *
 * Defines an interface for a generic customizer setting.
 *
 * @package VSHM
 */
abstract class SettingCustomizer implements Setting
{
    /**
     * @var string
     */
    public static $sanitizeCallback = '';

    /**
     * @return string
     */
    public static function getLabel(): string
    {
        return '';
    }

    /**
     * @return string
     */
    public static function getDescription(): string
    {
        return '';
    }

    /**
     * @return string
     */
    public static function getSection(): string
    {
        return '';
    }

    /**
     * @return string
     */
    public static function get_option_id(): string
    {
        return vshm()->get_settings_name() . '[style]' . '[' . static::ID . ']';
    }

    /**
     * @return array
     */
    public static function get_control(): array
    {
        return [];
    }

    public static function subscribe(): void
    {
        add_filter('vshm_default_plugin_settings', static function ($defaults) {
            $defaults['style'][ static::ID ] = static::getDefault();

            return $defaults;
        });
        add_action('vshm_settings_customizer', [static::class, 'customize_register'], 10, 2);
        add_filter('vshm_export_settings', static function ($settings) {
            $settings['style'][ static::ID ] = vshm()->settings()['style'][ static::ID ];

            return $settings;
        });
        add_filter('vshm_import_settings', static function ($toSave, $settings, $version) {
            if (isset($settings['style'][ static::ID ])) {
                $toSave['style'][ static::ID ] = static::whitelist($settings['style'][ static::ID ], $version);
            }

            return $toSave;
        }, 10, 3);
    }

    public static function whitelist($value, string $version)
    {
        return $value;
    }

    /**
     * @param        $wp_customize
     * @param string $plugin_slug
     */
    public static function customize_register($wp_customize, string $plugin_slug): void
    {
        if ($plugin_slug === vshm()->plugin['SLUG']) {

            $wp_customize->add_setting(static::get_option_id(), [
                'type'              => 'option',
                'capability'        => 'manage_options',
                'default'           => static::getDefault(),
                'transport'         => 'postMessage',
                'sanitize_callback' => static::$sanitizeCallback
            ]);
            $wp_customize->selective_refresh->add_partial(static::get_option_id(), [
                'selector'            => '.tbk-frontend',
                'container_inclusive' => FALSE,
                'render_callback'     => function () {
                    echo "<div class='tbk-inner-content'></div>";
                },
            ]);
            if (!empty(static::get_control())) {
                $wp_customize->add_control(static::get_option_id(), static::get_control());
            }

        }
    }

    public static function getBackendElement(): ?Element_Setting
    {
        return NULL;
    }

    /**
     * @return bool
     */
    public static function isPublic(): bool
    {
        return TRUE;
    }
}
